"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebSocketClientPeer = void 0;
const ataraxia_1 = require("ataraxia");
const AbstractWebSocketPeer_1 = require("./AbstractWebSocketPeer");
class WebSocketClientPeer extends AbstractWebSocketPeer_1.AbstractWebSocketPeer {
    constructor(parent, authProviders, factory, url) {
        super(parent, authProviders);
        this.factory = factory;
        this.url = url;
        this.backOff = new ataraxia_1.BackOff({
            delay: 100,
            maxDelay: 30000
        });
    }
    handleDisconnect(reason, err) {
        // If we are already waiting for a connection ignore this call
        if (this.connectTimeout)
            return;
        super.handleDisconnect(reason, err);
        const delay = this.backOff.nextDelay();
        this.debug('Reconnecting in', delay, 'ms');
        this.connectTimeout = setTimeout(() => this.tryConnect(), delay);
    }
    tryConnect() {
        // Make sure to clear the connection timeout when attempting to reconnect
        clearTimeout(this.connectTimeout);
        this.connectTimeout = null;
        // Create and setup the new WebSocket
        const ws = this.factory(this.url);
        ws.addEventListener('open', () => {
            this.debug('Connected');
            this.backOff.reset();
            this.negotiateAsClient();
        });
        this.setSocket(ws);
    }
    disconnect() {
        super.disconnect();
        clearTimeout(this.connectTimeout);
    }
}
exports.WebSocketClientPeer = WebSocketClientPeer;
//# sourceMappingURL=WebSocketClientPeer.js.map